#include "LiggghtsPanel.h"
#include "compareNames.h"
#include "ui_LiggghtsPanel.h"

#include <QDir>
#include <vtkSMProxy.h>
#include <pqApplicationCore.h>
#include <pqPVApplicationCore.h>
#include <pqServerManagerModel.h>
#include <vtkSMPropertyHelper.h>
#include <vtkSMProxyManager.h>
#include <pqTimeKeeper.h>
#include <pqObjectBuilder.h>
#include <pqServer.h>
#include <pqObjectInspectorWidget.h>
#include <pqAnimationManager.h>
#include <pqAnimationScene.h>

#include <pqDataRepresentation.h>
#include <pqView.h>
#include <vtkSMPVRepresentationProxy.h>

#include <pqActiveView.h>

#define QT_USE_FAST_CONCATENATION
#include <QStringBuilder>

class LiggghtsPanel::pqInternals : public Ui::LiggghtsPanel
{
public: 
 
};

void LiggghtsPanel::setRepresentation() {

 pqApplicationCore* core = pqApplicationCore::instance();
 pqObjectBuilder* builder = core->getObjectBuilder();

    pqActiveView& myActiveView=pqActiveView::instance();
    this->RenderView=myActiveView.current();
    pqDataRepresentation *repr = builder->createDataRepresentation(this->reader->getOutputPort(0), this->RenderView);
    vtkSMPropertyHelper(repr->getProxy(),"Representation").Set("Point Sprite");

     if(repr){
         //vtkSMPropertyHelper(repr->getProxy(), "Representation").Set(8);
         //pqDisplayPolicy pqDisplayP = core->getDisplayPolicy();
         //repr->getProxy()->UpdateVTKObjects();
	 printf("done\n");
     } else
	printf("repr. not found\n");

}


//-----------------------------------------------------------------------------
pqPipelineSource* LiggghtsPanel::NewSource(QStringList files,QString type)
{
  pqPipelineSource *src;
  src=NULL;
  pqApplicationCore* core = pqApplicationCore::instance();
  pqObjectBuilder* builder = core->getObjectBuilder();
  pqServerManagerModel* sm = core->getServerManagerModel();

  pqObjectInspectorWidget::setAutoAccept(true);
  // Check that we are connect to some server (either builtin or remote).
  if (sm->getNumberOfItems<pqServer*>())
    {
    // just create it on the first server connection
    pqServer* s = sm->getItemAtIndex<pqServer*>(0);
    
    src = builder->createReader(QString("sources"),type, files, s);
    src->rename(type);

    pqActiveView& myActiveView=pqActiveView::instance();
    this->RenderView=myActiveView.current();
    pqDataRepresentation *repr = builder->createDataRepresentation(src->getOutputPort(0), this->RenderView);
    //vtkSMPropertyHelper(repr->getProxy(),"Representation").Set("Point Sprite"); //segfault
    
    }
  pqObjectInspectorWidget::setAutoAccept(false);
  //setRepresentation();
  return src;
}

void LiggghtsPanel::constructor()
{
  this->reader=NULL;
  this->STLreader=NULL;

  this->foldername="./post/";
  this->setWindowTitle("Luke File Walker");
  QWidget* container = new QWidget(this);
  
  this->Internals = new pqInternals();
  this->Internals->setupUi(container);
  this->setWidget(container);

  watcher = new QFileSystemWatcher();
  watcher->addPath(this->foldername);  

  connect(watcher,SIGNAL(directoryChanged(QString)),this,SLOT(dirChangedSlot(QString)));
  dirChangedSlot(this->foldername);
}

void LiggghtsPanel::dirChangedSlot(QString path)
{
    if (!this->Internals->isActive->isChecked())
	return;
    //use QT to list dir
    QDir currentDir(path);
    QStringList filenames;
    QStringList filters;
    filters << "*.liggghts"; //"*.bin" << "*.liggghts" << "*.vtk"; //TODO add dropdownbox
    currentDir.setFilter(QDir::Files);
    currentDir.setNameFilters(filters);
    filenames = currentDir.entryList();
    if (filenames.size()>0) {
	    for (int i = 0; i < filenames.size(); ++i)
		filenames[i]=this->foldername % filenames.at(i);
	
	    qSort(filenames.begin(),filenames.end(),compareNames);
	    if (this->reader==NULL) {
		this->reader=NewSource(filenames,QString("liggghts_Reader"));
	    }
	    else {
	    vtkSMProxy* proxy=reader->getProxy();
	    vtkSMPropertyHelper(proxy, "FileNames").SetNumberOfElements(filenames.size());
	    for (int i = 0; i < filenames.size(); ++i)	       
	       vtkSMPropertyHelper(proxy, "FileNames").Set(i,filenames.at(i).toUtf8().constData());
	    proxy->UpdateVTKObjects();
	    }
	   
	    pqTimeKeeper* timekeeper = pqPVApplicationCore::instance()->animationManager()->
	   	getActiveScene()->getServer()->getTimeKeeper();
	    timekeeper->setTime(filenames.size());
		    
	    reader->renderAllViews(true);
	    reader->updatePipeline();
	    this->Internals->countlbl->setText(QString::number(filenames.size()));
    }

//STL
    filters.clear();
    filters << "*.stl"; //"*.bin" << "*.liggghts" << "*.vtk"; //TODO add dropdownbox
    currentDir.setFilter(QDir::Files);
    currentDir.setNameFilters(filters);
    filenames = currentDir.entryList();
    if (filenames.size()>0) {
	    qSort(filenames.begin(),filenames.end(),compareNames);
	    for (int i = 0; i < filenames.size(); ++i)
		filenames[i]=this->foldername % filenames.at(i);	
	    if (this->STLreader==NULL) {
		this->STLreader=NewSource(filenames,QString("stlreader"));
	    }
	    else {
	    vtkSMProxy* proxy=STLreader->getProxy();
	    vtkSMPropertyHelper(proxy, "FileNames").SetNumberOfElements(filenames.size());
	    for (int i = 0; i < filenames.size(); ++i)
	       vtkSMPropertyHelper(proxy, "FileNames").Set(i,filenames.at(i).toUtf8().constData());
	    proxy->UpdateVTKObjects();
	    }
	    
	    pqTimeKeeper* timekeeper = pqPVApplicationCore::instance()->animationManager()->
	   	getActiveScene()->getServer()->getTimeKeeper();
	    timekeeper->setTime(filenames.size());
		    
	    STLreader->renderAllViews(true);
	    STLreader->updatePipeline();
	    
    }
   
}
